
internal void
testWallCollision( vec2 playerPos, vec2 playerDelta, vec2 wallStartPoint, vec2 wallEndPoint, flo32 * tMin, vec2 * collisionNormal ) {
	vec2 wallVector = wallEndPoint - wallStartPoint;
	vec2 testNormal = getNormal( getPerp( wallVector ) );
	
	if( dot( playerDelta, testNormal ) < 0.0f ) {
		boo32 wallIsVertical = ( testNormal.y == 0.0f );
		if( wallIsVertical ) {
			flo32 testTMin = ( playerDelta.x != 0.0f ? ( wallStartPoint.x - playerPos.x ) / playerDelta.x : 2.0f );
			if( testTMin >= 0.0f && testTMin < *tMin ) {
				flo32 testY = playerPos.y + playerDelta.y * testTMin;
				flo32 minBound = ( wallStartPoint.y < wallEndPoint.y ? wallStartPoint.y : wallEndPoint.y );
				flo32 maxBound = ( wallStartPoint.y > wallEndPoint.y ? wallStartPoint.y : wallEndPoint.y );
				if( testY > minBound && testY < maxBound ) {
					*tMin = testTMin;
					*collisionNormal = testNormal;
				}
			}
		} else {
			flo32 testTMin = ( playerDelta.y != 0.0f ? ( wallStartPoint.y - playerPos.y ) / playerDelta.y : 2.0f );
			if( testTMin >= 0.0f && testTMin < *tMin ) {
				flo32 testX = playerPos.x + playerDelta.x * testTMin;
				flo32 minBound = ( wallStartPoint.x < wallEndPoint.x ? wallStartPoint.x : wallEndPoint.x );
				flo32 maxBound = ( wallStartPoint.x > wallEndPoint.x ? wallStartPoint.x : wallEndPoint.x );
				if( testX > minBound && testX < maxBound ) {
					*tMin = testTMin;
					*collisionNormal = testNormal;
				}
			}
		}
	}
}

internal boo32
rectCircleCollision( rect bound, vec2 circleCenter, flo32 circleRadius ) {
	boo32 result = false;
   rect boundA = { bound.min.x, bound.min.y - circleRadius, bound.max.x, bound.max.y + circleRadius };
	rect boundB = { bound.min.x - circleRadius, bound.min.y, bound.max.x + circleRadius, bound.max.y };
	flo32 circleRadiusSq = squareValue( circleRadius );

   if( isInBound( circleCenter, boundA ) || isInBound( circleCenter, boundB ) ||
      getLengthSq( circleCenter - bound.min ) < circleRadiusSq ||
      getLengthSq( circleCenter - Vec2( bound.max.x, bound.min.y ) ) < circleRadiusSq ||
      getLengthSq( circleCenter - Vec2( bound.min.x, bound.max.y ) ) < circleRadiusSq ||
      getLengthSq( circleCenter - bound.max ) < circleRadiusSq ) {
      result = true;
	}
	
	return result;
}

internal void
endReplayAttempt( APP_STATE * appState ) {
	uint32 attemptIndex = appState->replay_attemptCount;
	appState->replay_attemptData[ attemptIndex ].frameCount = appState->replay_currentFrameIndex - appState->replay_attemptData[ attemptIndex ].startIndex;
	appState->replay_attemptData[ attemptIndex ].endPosition = appState->player.position;
	
	appState->replay_attemptCount++;
	Assert( appState->replay_attemptCount < MAX_ATTEMPT );
	appState->replay_attemptData[ appState->replay_attemptCount ].startIndex = appState->replay_currentFrameIndex;
}
	
internal void
killPlayer( APP_STATE * appState ) {
	PLAYER_STATE * player = &appState->player;
	player->isDead = true;
	pushNewSplatter( &appState->splatterBuffer, player->position, appState->targetSec );
	endReplayAttempt( appState );
}

internal void
testEntityCollision( vec2 startPosition, vec2 moveVector, rect bound, flo32 * tMin, vec2 * collisionNormal ) {
	vec2 TL = Vec2( bound.left,  bound.top );
	vec2 TR = Vec2( bound.right, bound.top );
	vec2 BL = Vec2( bound.left,  bound.bottom );
	vec2 BR = Vec2( bound.right, bound.bottom );
	
	testWallCollision( startPosition, moveVector, BL, TL, tMin, collisionNormal );
	testWallCollision( startPosition, moveVector, BR, BL, tMin, collisionNormal );
	testWallCollision( startPosition, moveVector, TR, BR, tMin, collisionNormal );
	testWallCollision( startPosition, moveVector, TL, TR, tMin, collisionNormal );
}

internal vec2
clipCameraPosition( rect cameraBound, vec2 cameraPosition ) {
	cameraPosition.x = maxValue( cameraPosition.x, cameraBound.min.x );
	cameraPosition.x = minValue( cameraPosition.x, cameraBound.max.x );
	cameraPosition.y = maxValue( cameraPosition.y, cameraBound.min.y );
	cameraPosition.y = minValue( cameraPosition.y, cameraBound.max.y );
	return cameraPosition;
}

internal void
testMovement( boo32 * onGround, boo32 * onWall_left, boo32 * onWall_right, rect * bound_array, int32 bound_count, vec2 player_position, flo32 player_radius ) {
	for( int32 bound_index = 0; bound_index < bound_count; bound_index++ ) {
		rect    bound = bound_array[ bound_index ];
		rect adjBound = addRadius( bound, player_radius );
		
		if( ( player_position.y == adjBound.top   ) && ( player_position.x > adjBound.left   ) && ( player_position.x < adjBound.right ) ) {
			*onGround = true;
		}
		if( ( player_position.x == adjBound.right ) && ( player_position.y > adjBound.bottom ) && ( player_position.y < adjBound.top   ) ) {
			*onWall_left = true;
		}
		if( ( player_position.x == adjBound.left  ) && ( player_position.y > adjBound.bottom ) && ( player_position.y < adjBound.top   ) ) {
			*onWall_right = true;
		}
	}
}

internal void
initActiveEntities( APP_STATE * appState ) {
	uint32 currentLevel = appState->currentActiveLevel;
	uint32 entityCount = global_level_APP_STATE[ currentLevel ].entity_count;
	uint32 readIndex = global_level_APP_STATE[ currentLevel ].entity_baseIndex;
	
	GROUND_STATE * groundState = &appState->groundState;
	groundState->count = 0;
	
	CRUMBLE_STATE * crumbleState = &appState->crumbleState;
	crumbleState->count       = 0;
	crumbleState->timer_count = 0;
	
	SAWBLADE_STATE * sawbladeState = &appState->sawbladeState;
	sawbladeState->count          = 0;
	sawbladeState->moving_count   = 0;
	sawbladeState->rotating_count = 0;
	sawbladeState->launch_count   = 0;
	
	KILL_STATE * killState = &appState->killState;
	killState->count = 0;
	
	for( uint32 entityIndex = 0; entityIndex < entityCount; entityIndex++ ) {
		uint32 entityID = ( uint32 )global_entity_readData[ readIndex ];
		switch( entityID ) {
			case entityID_ground: {
				entity_groundData * readEntity = ( entity_groundData * )&global_entity_readData[ readIndex ];
				
				groundState->bound[ groundState->count++ ] = readEntity->terrainBound;
				Assert( groundState->count < MAX_GROUND_COUNT );
				
				readIndex += sizeof( entity_groundData ) / 4;
			} break;
			
			case entityID_crumble: {
				entity_crumble_readData * readEntity = ( entity_crumble_readData * )&global_entity_readData[ readIndex ];
				
				crumbleState->bound[ crumbleState->count++ ] = readEntity->terrainBound;
				Assert( crumbleState->count < CRUMBLE__MAX_COUNT );
				
				readIndex += sizeof( entity_crumble_readData ) / 4;
			} break;
			
			case entityID_kill: {
				entity_kill_readData * readEntity = ( entity_kill_readData * )&global_entity_readData[ readIndex ];
				
				killState->bound[ killState->count ] = readEntity->terrainBound;
				
				KILL * kill = killState->kill + killState->count;
				kill->counter = 0;
				for( int32 iter = 0; iter < 7; iter++ ) {
					kill->triangle[ iter ] = newTriangle();
				}
				
				killState->count++;
				Assert( killState->count < MAX_KILL_COUNT );
				
				readIndex += sizeof( entity_kill_readData ) / 4;
			} break;
			
			case entityID_sawBlade: {
				entity_sawBladeData * readEntity = ( entity_sawBladeData * )&global_entity_readData[ readIndex ];
				
				sawbladeState->sawblade[ sawbladeState->count++ ] = SawBlade( readEntity->position, readEntity->radius );
				
				readIndex += sizeof( entity_sawBladeData ) / 4;
			} break;
			
			case entityID_movingSawBlade: {
				entity_movingSawBlade_readData * readEntity = ( entity_movingSawBlade_readData * )&global_entity_readData[ readIndex ];
				
				if( sawbladeState->moving_count == 0 ) {
					sawbladeState->moving_baseIndex = sawbladeState->count;
				}
				sawbladeState->sawblade[ sawbladeState->count++        ] = SawBlade( readEntity->position, readEntity->radius );
				sawbladeState->moving  [ sawbladeState->moving_count++ ] = MovingState( readEntity->moveVector_perSecond, readEntity->targetHalfCycleTime );
				
				readIndex += sizeof( entity_movingSawBlade_readData ) / 4;
			} break;
			
			case entityID_rotatingSawBlade: {
				entity_rotatingSawBlade_readData * readEntity = ( entity_rotatingSawBlade_readData * )&global_entity_readData[ readIndex ];
				
				if( sawbladeState->rotating_count == 0 ) {
					sawbladeState->rotating_baseIndex = sawbladeState->count;
				}
				sawbladeState->sawblade[ sawbladeState->count++          ] = SawBlade( readEntity->centerPosition + Vec2( readEntity->armRadius, 0.0f ), readEntity->bladeRadius );
				sawbladeState->rotating[ sawbladeState->rotating_count++ ] = RotatingState( readEntity->centerPosition, ( -1.0f / readEntity->radiansPerSecond * 2.0f * PI ), readEntity->armRadius );
				
				readIndex += sizeof( entity_rotatingSawBlade_readData ) / 4;
			} break;
			
			case entityID_flyingSawBlade: {
				entity_flyingSawBlade_readData * readEntity = ( entity_flyingSawBlade_readData * )&global_entity_readData[ readIndex ];
				readIndex += sizeof( entity_flyingSawBlade_readData ) / 4;
				
				uint32 sawBladeCount = __reinterpretToUInt( readEntity->sawBladeCount );
				
				sawbladeState->sawblade[ sawbladeState->count++ ] = SawBlade( readEntity->startPosition, readEntity->radius );
				Assert( sawbladeState->count < SAWBLADE__MAX_COUNT );
				
				LAUNCH_STATE * launchState = sawbladeState->launch + sawbladeState->launch_count++;
				Assert( sawbladeState->launch_count < SAWBLADE__MAX_LAUNCH_STATE_COUNT );
				launchState->position     = readEntity->startPosition;
				launchState->velocity     = readEntity->moveVector_perSecond;
				launchState->launch_timer = readEntity->targetLaunchTime - 0.666666687f;
				launchState->launchTime   = readEntity->targetLaunchTime;
				launchState->flightTime   = readEntity->targetFlightTime;
				
				int32 sawblade_count = ( int32 )sawBladeCount;
				launchState->count     = sawblade_count;
				launchState->baseIndex = sawbladeState->count;
				launchState->currIndex = 0;
				for( int32 iter = 0; iter < sawblade_count; iter++ ) {
					launchState->flight_timer[ iter ] = -1000000.0f;
					sawbladeState->sawblade[ sawbladeState->count++ ] = SawBlade( Vec2( 0.0f, -1000000.0f ), readEntity->radius );
					Assert( sawbladeState->count < SAWBLADE__MAX_COUNT );
				}
			} break;
		}
	}
}

internal void
updateActiveEntities( APP_STATE * appState ) {
	updateTimer(    &appState->crumbleState,  appState->targetSec );
	updateMoving(   &appState->sawbladeState, appState->targetSec );
	updateRotating( &appState->sawbladeState, appState->targetSec );
	updateFlying(   &appState->sawbladeState, appState->targetSec );
	update( &appState->killState );
}

internal void
drawActiveEntities( BACKBUFFER * backbuffer, APP_STATE * appState ) {
	VIEWPORT * camera = &appState->camera;
	drawBox( backbuffer, camera, global_level_APP_STATE[ appState->currentActiveLevel ].goalBound, COLOR_BLACK, COLOR_GOAL );
	
	draw( backbuffer, camera, &appState->sawbladeState );
	draw( backbuffer, camera, &appState->groundState   );
	draw( backbuffer, camera, &appState->crumbleState  );
	draw( backbuffer, camera, &appState->killState     );
}

internal void
resetLevel( APP_STATE * appState ) {
	PLAYER_STATE * player = &appState->player;
	SAVE_STATE * saveState = &appState->saveState;
	
	initActiveEntities( appState );
	player->isDead = false;
	player->velocity_perSecond = Vec2( 0.0f, 0.0f );
	
	appState->level_currentTime = 0.0f;
	player->position = global_level_APP_STATE[ appState->currentActiveLevel ].spawnPosition;
	appState->level_reloadTimer = 0.0f;
	
	appState->fadeIn = true;
	
	if( ( appState->currentActiveLevel == 0 ) && ( saveState->tutorial_levelOne_isActive ) ) {
		player->position.y += 400.0f;
	}
	if( ( appState->currentActiveLevel == 1 ) && ( saveState->tutorial_levelTwo_isActive ) ) {
		appState->tutorial_timer = 0.0f;
		appState->tutorial_toggleState = true;
		appState->tutorial_mode = 0;
	}
}

internal void
initLevel( APP_STATE * appState, uint32 levelToInit ) {
	appState->appMode = appMode_mainGame;
	appState->currentActiveLevel = levelToInit;
	
	appState->replay_currentFrameIndex = 0;
	appState->replay_attemptCount = 0;
	
	appState->splatterBuffer.count = 0;
	appState->splatterBuffer.startIndex = 0;
	appState->splatterBuffer.endIndex = 0;
	
	resetLevel( appState );
}

internal void
resetReplay( APP_STATE * appState ) {
	appState->replay_currentFrameIndex = 0;
	initActiveEntities( appState );
}

internal void
initReplay( APP_STATE * appState ) {
	appState->appMode = appMode_levelReplay;
	if( appState->saveState.tutorial_replay_isActive ) {
		appState->tutorial_timer = 0.20f;
		appState->tutorial_toggleState = false;
	}
	resetReplay( appState );
}

internal void
drawTimeText( BACKBUFFER * backbuffer, APP_STATE * appState ) {
	flo32 backbufferHeight = ( flo32 )backbuffer->height;
	flo32 colon_xPosition = appState->app_dim.x - 160.0f;
	flo32 verticalSpacing = appState->font_med->advanceHeight;
	
	vec2 atPos = { colon_xPosition - 20.0f, backbufferHeight - 10.0f - appState->font_med->ascent };
	
	char timeText[32] = {};
	sprintf( timeText, "Time: %.02f", appState->level_currentTime );
	drawString( backbuffer, appState->font_med, timeText, atPos );
	
	flo32 bestTime = global_level_APP_STATE[ appState->currentActiveLevel ].bestTime;
	flo32 targetTime = global_level_APP_STATE[ appState->currentActiveLevel ].targetTime;
	if( bestTime == 0.0f ) {
		char targetTimeText[32] = {};
		sprintf( targetTimeText, "Target Time: %.02f", targetTime );
		vec2 targetTimePosition = atPos - Vec2( 106.0f, verticalSpacing );
		drawString( backbuffer, appState->font_med, targetTimeText, targetTimePosition );
	} else if( bestTime > targetTime ) {
		char targetTimeText[32] = "";
		sprintf( targetTimeText, "Target Time: %.02f", targetTime );
		vec2 targetTimePosition = atPos - Vec2( 106.0f, verticalSpacing );
		drawString( backbuffer, appState->font_med, targetTimeText, targetTimePosition );
		
		char bestAttemptText[32] = "";
		sprintf( bestAttemptText, "Best Attempt: %.02f", bestTime );
		vec2 bestAttemptPosition = atPos - Vec2( 122.0f, verticalSpacing * 2.0f );
		drawString( backbuffer, appState->font_med, bestAttemptText, bestAttemptPosition );
	} else {
		char bestTimeText[32] = "";
		sprintf( bestTimeText, "Best Time: %.02f", bestTime );
		vec2 bestTimePosition = atPos - Vec2( 83.0f, verticalSpacing );
		drawString( backbuffer, appState->font_med, bestTimeText, bestTimePosition );
	}
}

internal void
setAppMode( APP_STATE * appState, APP_MODE newMode ) {
	appState->appMode = newMode;
	appState->appMode_isInitialized = false;
}

internal void
updateFadeIn( BACKBUFFER * backbuffer, APP_STATE * appState ) {
	if( appState->fadeIn ) {
		flo32 ratio = minValue( ( appState->fadeIn_timer / 0.2f ), 1.0f );
		
		rect app_bound = appState->app_bound;
		rect bound = Rect( ratio * appState->app_dim.x, app_bound.bottom, app_bound.right, app_bound.top );
		
		drawRect( backbuffer, bound, COLOR_BLACK );
		
		if( appState->fadeIn_timer <= 0.2f ) {
			appState->fadeIn_timer += appState->targetSec;	
		} else {
			appState->fadeIn = false;
			appState->fadeIn_timer = 0.0f;
		}
	}
}

internal void
updateFadeOut( BACKBUFFER * backbuffer, APP_STATE * appState ) {
	if( appState->fadeOut ) {
		flo32 ratio = minValue( ( appState->fadeIn_timer / 0.2f ), 1.0f );
		
		rect app_bound = appState->app_bound;
		rect bound = Rect( app_bound.left, app_bound.bottom, ( appState->fadeOut_timer / 0.2f ) * appState->app_dim.x, app_bound.top );
		
		drawRect( backbuffer, bound, COLOR_BLACK );
		
		if( appState->fadeOut_timer <= 0.2f ) {
			appState->fadeOut_timer += appState->targetSec;
		} else {
			appState->fadeOut = false;
			appState->fadeOut_timer = 0.0f;
			appState->exitMode = true;
		}
	}
}

internal void
tutorial_levelOne( BACKBUFFER * backbuffer, APP_STATE * appState ) {
	PLAYER_STATE * player = &appState->player;
	VIEWPORT     * camera = &appState->camera;
	SAVE_STATE   * saveState = &appState->saveState;
	
	FONT * font_med = appState->font_med;
	if( ( saveState->tutorial_levelOne_isActive ) && ( appState->currentActiveLevel == 0 ) ) {
		switch( appState->tutorial_mode ) {
			case 0: {
				if( isInBound( player->position, rectCR( Vec2( 252.0f, 120.0f ), 48.0f ) ) ) {
					appState->tutorial_mode++;
				}
			} break;
			
			case 1: {
				if( !isInBound( player->position, rectCR( Vec2( 252.0f, 120.0f ), 48.0f ) ) ) {
					appState->tutorial_mode++;
				}
				rect moveTextBound = rectTLD( Vec2( -20.0f, 240.0f ), Vec2( 492.0f, 94.0f ) );
				drawBox( backbuffer, camera, moveTextBound, COLOR_BLACK, COLOR_WHITE );
				
				drawString( backbuffer, font_med, "LEFT ARROW : Move Left",   camera, getBL( moveTextBound ) + Vec2( 14.0f, 12.0f ) );
				drawString( backbuffer, font_med, "RIGHT ARROW : Move Right", camera, getBL( moveTextBound ) + Vec2( 14.0f, 50.0f ) );
			} break;
			
			case 2: {
				rect jumpTut_bound = rectBLD( Vec2( 158.0f, 82.0f ), Vec2( 188.0f, 108.0f ) );
				if( !isInBound( player->position, jumpTut_bound ) ) {
					appState->tutorial_mode++;
					appState->tutorial_toggleState = false;
					appState->tutorial_timer = 0.0f;
				}
				rect jumpTextBound = rectCD( Vec2( 352.0f, 204.0f ), Vec2( 348.0f, 54.0f ) );
				drawBox( backbuffer, camera, jumpTextBound, COLOR_BLACK, COLOR_WHITE );
				drawString( backbuffer, font_med, "SPACEBAR : Jump", camera, getBL( jumpTextBound ) + Vec2( 14.0f, 12.0f ) );
			} break;
			
			case 3: {
				appState->tutorial_timer += appState->targetSec;
				if( appState->tutorial_timer >= 0.666666687f ) {
					appState->tutorial_timer -= 0.666666687f;
					appState->tutorial_toggleState = !appState->tutorial_toggleState;
				}
				if( appState->tutorial_toggleState ) {
					vec2 goalBasePoint = Vec2( -60.0f, 532.0f );
					
					vec2 stringPos = goalBasePoint + Vec2( -6.0f, 2.0f );
					drawString( backbuffer, font_med, "Move here!", camera, stringPos );
					drawLine( backbuffer, camera, goalBasePoint, goalBasePoint + Vec2( 156.0f, 0.0f ), COLOR_BLACK );
					drawLine( backbuffer, camera, goalBasePoint + Vec2( 60.0f, 0.0f ), goalBasePoint + Vec2( 60.0f, -20.0f ), COLOR_BLACK );
					drawLine( backbuffer, camera, goalBasePoint + Vec2( 60.0f, -20.0f ), goalBasePoint + Vec2( 148.0f, -38.0f ), COLOR_BLACK );
				}
			} break;
		}
	}
}

internal void
tutorial_levelTwo( BACKBUFFER * backbuffer, APP_STATE * appState ) {
	PLAYER_STATE * player   = &appState->player;
	VIEWPORT     * camera   = &appState->camera;
	FONT         * font_med =  appState->font_med;

	if( ( appState->saveState.tutorial_levelTwo_isActive ) && ( appState->currentActiveLevel == 1 ) ) {
		appState->tutorial_timer += appState->targetSec;
		if( appState->tutorial_timer >= 1.33333337f ) {
			appState->tutorial_timer -= 1.33333337f;
			appState->tutorial_toggleState = !appState->tutorial_toggleState;
		}
		
		vec2 goalTextBasePosition = Vec2( 1080.0f, 360.0f );
		drawLine( backbuffer, camera, goalTextBasePosition, goalTextBasePosition + Vec2( 112.0f, 0.0f ), COLOR_BLACK );
		drawLine( backbuffer, camera, goalTextBasePosition + Vec2( 92.0f, 0.0f ), goalTextBasePosition + Vec2( 104.0f, -18.0f ), COLOR_BLACK );
		if( appState->tutorial_toggleState ) {
			drawString( backbuffer, font_med, "Run...", camera, goalTextBasePosition + Vec2( -46.0f, 2.0f ) );
		} else {
			drawString( backbuffer, font_med, "Jump!!", camera, goalTextBasePosition + Vec2( 50.0f, 2.0f ) );
		}
		switch( appState->tutorial_mode ) {
			case 0: {
				if( isInBound( player->position, rectTLD( Vec2( 216.0f, 360.0f ), Vec2( 288.0f, 48.0f ) ) ) ) {
					appState->tutorial_mode++;
				}
			} break;
			
			case 1: {
				rect moveTut_bound = rectTLD( Vec2( 72.0f, 360.0f ), Vec2( 432.0f, 48.0f ) );
				if( isInBound( player->position, moveTut_bound ) ) {
					rect moveFastTextBound = rectTLD( Vec2( 144.0f, 436.0f ), Vec2( 412.0f, 54.0f ) );
					drawBox( backbuffer, camera, moveFastTextBound, COLOR_BLACK, COLOR_WHITE );
					drawString( backbuffer, font_med, "HOLD SHIFT : Move Fast", camera, moveFastTextBound.min + Vec2( 14.0f, 12.0f ) );
				}
			} break;
		}
		
		vec2 speedBasePosition = { 216.0f, 288.0f };
		drawCircle( backbuffer, camera, speedBasePosition, 8.0f, COLOR_BLACK, COLOR_BLACK );
		if( appState->fast_isDown ) {
			drawCircle( backbuffer, camera, speedBasePosition, 5.0f, COLOR_YELLOW, COLOR_YELLOW );
		}
		
		rect meterOutlineBound = rectMM( speedBasePosition + Vec2( 14.0f, -6.0f ), Vec2( 494.0f, speedBasePosition.y + 6.0f ) );
		drawRect( backbuffer, camera, meterOutlineBound, COLOR_BLACK );
		
		rect meterBound = addRadius( meterOutlineBound, -3.0f );
		flo32 meterLength = meterBound.max.x - meterBound.min.x;
		flo32 meterHalfLength = meterLength * 0.5f;
		
		flo32 velocityNormal = absValue( player->velocity_perSecond.x ) / 800.0f;
		flo32 boxALength = meterLength * velocityNormal;
		flo32 boxBLength = minValue( maxValue( boxALength - meterHalfLength, 0.0f ), meterHalfLength );
		vec2 boxAPosition = meterBound.min;
		vec2 boxBPosition = boxAPosition + Vec2( meterHalfLength, 0.0f );
		drawRect( backbuffer, camera, rectBLD( boxAPosition, Vec2( boxALength, 6.0f ) ), COLOR_GREEN );
		drawRect( backbuffer, camera, rectBLD( boxBPosition, Vec2( boxBLength, 6.0f ) ), COLOR_RED   );
	}
}

internal boo32
tutorial_replay( BACKBUFFER * backbuffer, APP_STATE * appState ) {
	FONT * font_lg = appState->font_lg;
	
	boo32 result = false;
	if( appState->saveState.tutorial_replay_isActive ) {
		if( appState->tutorial_toggleState ) {
			result = true;
			
			vec2 stringPos = { appState->app_halfDim.x, appState->app_dim.y - 100.0f };
			drawStringCentered( backbuffer, font_lg, "Instant Replay", stringPos );
		}
		appState->tutorial_timer += appState->targetSec;
		if( appState->tutorial_timer >= 1.0f ) {
			appState->tutorial_timer -= 1.0f;
			
			appState->tutorial_toggleState = !appState->tutorial_toggleState;
		}
	}
	return result;
}

internal void
saveGame( PLATFORM * platform, SAVE_STATE * saveState ) {
	platform->writeFile( "save.dat", saveState, sizeof( SAVE_STATE ) );
}